# Eve IDs
EVE_CATEGORY_ID_STATION = 3
EVE_CATEGORY_ID_SHIP = 6
EVE_CATEGORY_ID_MODULE = 7
EVE_CATEGORY_ID_CHARGE = 8
EVE_CATEGORY_ID_BLUEPRINT = 9
EVE_CATEGORY_ID_SKILL = 16
EVE_CATEGORY_ID_DRONE = 18
EVE_CATEGORY_ID_IMPLANT = 20
EVE_CATEGORY_ID_FIGHTER = 87
EVE_CATEGORY_ID_STRUCTURE = 65
EVE_TYPE_ID_SOLAR_SYSTEM = 5
