{% load i18n %}

<div role="tabpanel" class="tab-pane" id="assets">
    <div class="table-responsive">
        <table class="table table-striped table-width-fix" id="tab_assets">
            <thead>
                <tr>
                    <th>{% translate 'Location' %}</th>
                    <th>{% translate 'Name' %}</th>
                    <th>{% translate 'Qty' %}</th>
                    <th>{% translate 'Group' %}</th>
                    <th>{% translate 'Volume (m3)' %}</th>
                    <th>{% translate 'Price' %}</th>
                    <th>{% translate 'Total' %}</th>
                    <th></th>
                </tr>
            </thead>

            <tbody></tbody>

            <tfoot>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th>{% translate 'Grand Total:' %}</th>
                <th></th>
                <th></th>
            </tfoot>
        </table>
    </div>

    {% include 'memberaudit/partials/character_viewer/tabs/last_updated.html' with update_section=last_updates.assets %}
    <p class="text-muted">{% translate 'Prices are average market prices.' %}</p>
</div>
