{% load i18n %}
{% load static %}
{% load humanize %}
{% load evelinks %}

<div class="modal" id="modalCharacterSkillSetDetails" tabindex="-1" role="dialog" aria-labelledby="myModalLabel">
    <div class="modal-dialog modal-lg" role="document" id="modalCharacterSkillSetContents">
        <div class="modal-content">
            <div class="modal-header">
                <button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">
                        &times;
                    </span>
                </button>
                <h4 class="modal-title" id="myModalLabel">{% translate 'Loading...' %}</h4>
            </div>

            <div class="modal-body">
                <div class="table-responsive">
                    <table class="table table-striped table-width-fix" id="skill_set_details">
                        <thead>
                            <tr>
                                <th>{% translate 'Name' %}</th>
                                <th class="text-center">{% translate 'Required' %}</th>
                                <th class="text-center">{% translate 'Recommended' %}</th>
                                <th class="text-center">{% translate 'Current' %}</th>
                                <th></th>
                            </tr>
                        </thead>

                        <tbody></tbody>
                    </table>
                </div>
            </div>

            <div class="modal-footer">
                <button type="button" class="btn btn-default" data-dismiss="modal">
                    {% translate 'Close' %}
                </button>
            </div>
        </div>
    </div>
</div>
