 {% extends 'admin/base_site.html' %}

 {% block content %}
 <form action="" method="post">
    {% csrf_token %}
    <div>
        <ul>
        {% for character in queryset %}
            <input type="hidden" name="_selected_action" value="{{ character.pk }}" />
            <li>{{ character.name }} </li>
        {% endfor %}
        </ul>
        <br>
        <input type="hidden" name="action" value="delete_characters" />
        <input type="submit" name="apply" value="Delete" style="background-color: rgb(201, 48, 44)"/>
        <a href="./"><input type="button" name="Cancel" value="Cancel"></a>
    </div>
 </form>
 {% endblock %}
